using System;
using System.IO;
using System.Xml.Serialization;
using UnityEditor;
using UnityEngine;

public class SettingsLoader {

  public IRenderSettingPreset Load() {

    string path = EditorUtility.OpenFilePanel("Choose file to load",  "","json");

    if (File.Exists(path)) {
      string dataAsJson = File.ReadAllText(path);
      Debug.Log("Loaded JSON: " + dataAsJson);
      PresetReferenceData presetData = JsonUtility.FromJson<PresetReferenceData>(dataAsJson);

      RenderSettingPreset preset = new RenderSettingPreset();
      preset.SetName(presetData.GetName());

      string relativePath = CalculateRelativePath(Directory.GetParent(path).Name);

      foreach(VisualBlueprintReferenceData blueprint in presetData.GetVisualBlueprintData()) {
        preset.AddVisualBlueprint(LoadVisualBlueprint(relativePath, blueprint));
      }

      return preset;
    } else {
      return null;
    }
  }

  private string CalculateRelativePath(string path) {
    Debug.Log(path);
    return "Assets/RenderPresets/" + path + "/";
  }

  private VisualBlueprint LoadVisualBlueprint(string presetDirectory , VisualBlueprintReferenceData data) {
    string name = data.GetName();

    Debug.Log("Preset Directory name: " + presetDirectory);


    //Interactive mode is set to false because the dialogue box is not needed
    AssetDatabase.ImportPackage(presetDirectory + data.GetDependenciesPath(), false);

    GameObject blueprintObject = LoadBlueprintObject(presetDirectory +  data.GetBlueprintObjectPath());

    return new VisualBlueprint(name, blueprintObject);
  }

  private GameObject LoadBlueprintObject(string path) {
    return (GameObject)AssetDatabase.LoadAssetAtPath(path, typeof(GameObject));
  }
}
