using System.Linq;
using System.Collections.Generic;
using System.IO;
using System.Xml.Serialization;
using UnityEditor;
using UnityEngine;

public class SettingsSaver : ISettingsSaver {

  private const string presetsRoot = "Assets/RenderPresets/";
  private const string visualBlueprintFolder = "VisualBlueprints/";
  private const string presetConfExtension = ".json";

  public void Save(IRenderSettingPreset preset) {

      CreateRequiredFolders(preset);

      SavePreset(preset);
  }

  private void CreateRequiredFolders(IRenderSettingPreset preset) {

    if (!Directory.Exists("Assets/RenderPresets/" + preset.GetName() + "/")) {
      Directory.CreateDirectory("Assets/RenderPresets/" + preset.GetName() + "/");
      Directory.CreateDirectory("Assets/RenderPresets/" + preset.GetName() + "/VisualBlueprints/");
    }
  }

  private void SavePreset(IRenderSettingPreset preset) {

    PresetReferenceData presetData = new PresetReferenceData(preset.GetName());

    foreach(VisualBlueprint blueprint in preset.GetVisualBlueprints()) {
      string savedPrefabPath = SaveVisualBlueprint(blueprint, preset);
      string dependencyPath = ExportDependencies(blueprint, preset);
      presetData.AddBlueprintReferenceData(new VisualBlueprintReferenceData(blueprint.GetName(), savedPrefabPath, dependencyPath));
    }

    string presetFile =  "Assets/RenderPresets/" + preset.GetName() + "/" + "presetconfig" + presetConfExtension;
    string serializedPresetData = JsonUtility.ToJson(presetData);
    File.WriteAllText(presetFile, serializedPresetData);

  }

  private string SaveVisualBlueprint(VisualBlueprint blueprint, IRenderSettingPreset preset) {
    //Path of blueprint relative to prese root

    PrefabUtility.CreatePrefab("Assets/RenderPresets/" + preset.GetName() + "/VisualBlueprints/" + blueprint.GetName() + ".prefab", blueprint.GetBlueprintObject());
    // PrefabUtility.ReplacePrefab(blueprint.GetBlueprintObject(), prefab, ReplacePrefabOptions.ConnectToPrefab);

    return "VisualBlueprints/" + blueprint.GetName() + ".prefab";
  }

  private string ExportDependencies(VisualBlueprint blueprint, IRenderSettingPreset preset) {

    string[] dependencies = AssetDatabase.GetDependencies("Assets/RenderPresets/" + preset.GetName() + "/VisualBlueprints/" + blueprint.GetName() + ".prefab");

    AssetDatabase.ExportPackage(dependencies, "Assets/RenderPresets/" + preset.GetName() + "/VisualBlueprints/" + blueprint.GetName() + "_dep.unitypackage");

    return "VisualBlueprints/" + blueprint.GetName() + "_dep.unitypackage";
  }

  private string GetFilePath(IRenderSettingPreset preset) {
    return EditorUtility.SaveFilePanel("Choose location to save file",  "", preset.GetName() + ".json","json");
  }
}
